@@ -113,8 +113,6 @@
 //!
 //! Some additional filters are available in the
 //! [`minijinja-contrib`](https://crates.io/crates/minijinja-contrib) crate.
-use std::sync::Arc;
-
 use crate::error::Error;
 use crate::utils::write_escaped;
 use crate::value::Value;
@@ -169,12 +167,14 @@ mod builtins {
 
     use crate::error::ErrorKind;
     use crate::utils::{safe_sort, splitn_whitespace};
-    use crate::value::ops::{self, as_f64};
-    use crate::value::{Enumerator, Kwargs, Object, ObjectRepr, ValueKind, ValueRepr};
+    use crate::value::merge_object::{MergeDict, MergeSeq};
+    use crate::value::ops::{self, as_f64, LenIterWrap};
+    use crate::value::{Enumerator, Kwargs, Object, ObjectRepr, Rest, ValueKind, ValueRepr};
     use std::borrow::Cow;
     use std::cmp::Ordering;
     use std::fmt::Write;
     use std::mem;
+    use std::sync::Arc;
 
     /// Converts a value to uppercase.
     ///
@@ -376,7 +376,20 @@ mod builtins {
         v.reverse()
     }
 
-    /// Trims a value
+    /// Trims a string.
+    ///
+    /// By default, it trims leading and trailing whitespaces:
+    ///
+    /// ```jinja
+    /// {{ "  non-space characters  " | trim }} -> "non-space characters"
+    /// ```
+    ///
+    /// You can also remove a character sequence.  All the prefixes and suffixes
+    /// matching the sequence are removed:
+    ///
+    /// ```jinja
+    /// {{ "1212foo12bar1212" | trim("12") }} -> "foo12bar"
+    /// ```
     #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn trim(s: Cow<'_, str>, chars: Option<Cow<'_, str>>) -> String {
         match chars {
@@ -409,8 +422,8 @@ mod builtins {
         }));
 
         let mut rv = String::new();
-        for item in iter {
-            if !rv.is_empty() {
+        for (idx, item) in iter.enumerate() {
+            if idx > 0 {
                 rv.push_str(joiner);
             }
             if let Some(s) = item.as_str() {
@@ -1028,7 +1041,7 @@ mod builtins {
                     output.push_str(&indent_with);
                 }
             } else {
-                write!(output, "{}{}", indent_with, line).ok();
+                write!(output, "{indent_with}{line}").ok();
             }
             output.push('\n');
         }
@@ -1467,12 +1480,139 @@ mod builtins {
         Ok(Value::from(rv))
     }
 
+    /// Chain two or more iterable objects as a single iterable object.
+    ///
+    /// If all the individual objects are dictionaries, then the final chained object
+    /// also acts like a dictionary -- you can lookup a key, or iterate over the keys
+    /// etc. Note that the dictionaries are not merged, so if there are duplicate keys,
+    /// then the lookup will return the value from the last matching dictionary in the
+    /// chain.
+    ///
+    /// If all the individual objects are sequences, then the final chained
+    /// object also acts like a list as if the lists are appended.
+    ///
+    /// Otherwise, the chained object acts like an iterator chaining individual
+    /// iterators, but it cannot be indexed.
+    ///
+    /// ```jinja
+    /// {{ users | chain(moreusers) | length }}
+    /// {% for user, info in shard0 | chain(shard1, shard2) | dictsort %}
+    ///   {{user}}: {{info}}
+    /// {% endfor %}
+    /// {{ list1 | chain(list2) | attr(1) }}
+    /// ```
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn chain(
+        _state: &State,
+        value: Value,
+        others: crate::value::Rest<Value>,
+    ) -> Result<Value, Error> {
+        let all_values = Some(value.clone())
+            .into_iter()
+            .chain(others.0.iter().cloned())
+            .collect::<Vec<_>>();
+
+        if all_values.iter().all(|v| v.kind() == ValueKind::Map) {
+            Ok(Value::from_object(MergeDict::new(all_values)))
+        } else if all_values
+            .iter()
+            .all(|v| matches!(v.kind(), ValueKind::Seq))
+        {
+            Ok(Value::from_object(MergeSeq::new(all_values)))
+        } else {
+            // General iterator chaining behavior
+            Ok(Value::make_object_iterable(all_values, |values| {
+                Box::new(values.iter().flat_map(|v| match v.try_iter() {
+                    Ok(iter) => Box::new(iter) as Box<dyn Iterator<Item = Value> + Send + Sync>,
+                    Err(err) => Box::new(Some(Value::from(err)).into_iter())
+                        as Box<dyn Iterator<Item = Value> + Send + Sync>,
+                })) as Box<dyn Iterator<Item = Value> + Send + Sync>
+            }))
+        }
+    }
+
+    /// Zip multiple iterables into tuples.
+    ///
+    /// This filter works like the Python `zip` function. It takes one or more
+    /// iterables and returns an iterable of tuples where each tuple contains
+    /// one element from each input iterable. The iteration stops when the
+    /// shortest iterable is exhausted.
+    ///
+    /// ```jinja
+    /// {{ [1, 2, 3]|zip(['a', 'b', 'c']) }}
+    /// -> [(1, 'a'), (2, 'b'), (3, 'c')]
+    ///
+    /// {{ [1, 2]|zip(['a', 'b', 'c'], ['x', 'y', 'z']) }}
+    /// -> [(1, 'a', 'x'), (2, 'b', 'y')]
+    /// ```
+    #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
+    pub fn zip(_state: &State, value: Value, others: Rest<Value>) -> Result<Value, Error> {
+        let all_values = Some(value).into_iter().chain(others.0).collect::<Vec<_>>();
+
+        // Validate all values are iterable and calculate minimum length
+        let mut known_len: Option<usize> = None;
+        for val in &all_values {
+            match val.try_iter() {
+                Ok(_) => {
+                    // If all values have known lengths, track the minimum
+                    if let Some(len) = val.len() {
+                        known_len = Some(match known_len {
+                            None => len,
+                            Some(current_min) => current_min.min(len),
+                        });
+                    } else {
+                        // If any value doesn't have a known length, we can't know the zip length
+                        known_len = None;
+                        break;
+                    }
+                }
+                Err(_) => {
+                    return Err(Error::new(
+                        ErrorKind::InvalidOperation,
+                        format!("zip filter argument must be iterable, got {}", val.kind()),
+                    ));
+                }
+            }
+        }
+
+        Ok(Value::make_object_iterable(all_values, move |values| {
+            let iter = std::iter::from_fn({
+                let mut iters = values
+                    .iter()
+                    .map(|val| val.try_iter().ok())
+                    .collect::<Option<Vec<_>>>()
+                    .unwrap_or_default();
+
+                move || {
+                    if iters.is_empty() {
+                        return None;
+                    }
+
+                    let mut tuple = Vec::with_capacity(iters.len());
+                    for iter in &mut iters {
+                        match iter.next() {
+                            Some(val) => tuple.push(val),
+                            None => return None,
+                        }
+                    }
+                    Some(Value::from(tuple))
+                }
+            });
+
+            if let Some(len) = known_len {
+                Box::new(LenIterWrap(len, iter)) as Box<dyn Iterator<Item = Value> + Send + Sync>
+            } else {
+                Box::new(iter) as Box<dyn Iterator<Item = Value> + Send + Sync>
+            }
+        }))
+    }
+
     /// Pretty print a variable.
     ///
     /// This is useful for debugging as it better shows what's inside an object.
     #[cfg_attr(docsrs, doc(cfg(feature = "builtins")))]
     pub fn pprint(value: &Value) -> String {
-        format!("{:#?}", value)
+        format!("{value:#?}")
     }
 }
 
