@@ -3,6 +3,23 @@ use crate::value::{DynObject, ObjectRepr, Value, ValueKind, ValueRepr};
 
 const MIN_I128_AS_POS_U128: u128 = 170141183460469231731687303715884105728;
 
+/// Iterator wrapper that provides exact size hints for iterators with known length.
+pub(crate) struct LenIterWrap<I: Send + Sync>(pub(crate) usize, pub(crate) I);
+
+impl<I: Iterator<Item = Value> + Send + Sync> Iterator for LenIterWrap<I> {
+    type Item = Value;
+
+    #[inline(always)]
+    fn next(&mut self) -> Option<Self::Item> {
+        self.1.next()
+    }
+
+    #[inline(always)]
+    fn size_hint(&self) -> (usize, Option<usize>) {
+        (self.0, Some(self.0))
+    }
+}
+
 pub enum CoerceResult<'a> {
     I128(i128, i128),
     F64(f64, f64),
@@ -62,20 +79,21 @@ pub fn coerce<'x>(a: &'x Value, b: &'x Value, lossy: bool) -> Option<CoerceResul
 }
 
 fn get_offset_and_len<F: FnOnce() -> usize>(
-    start: i64,
+    start: Option<i64>,
     stop: Option<i64>,
     end: F,
 ) -> (usize, usize) {
+    let start = start.unwrap_or(0);
     if start < 0 || stop.map_or(true, |x| x < 0) {
         let end = end();
         let start = if start < 0 {
-            (end as i64 + start) as usize
+            std::cmp::max(0, end as i64 + start) as usize
         } else {
             start as usize
         };
         let stop = match stop {
             None => end,
-            Some(x) if x < 0 => (end as i64 + x) as usize,
+            Some(x) if x < 0 => std::cmp::max(0, end as i64 + x) as usize,
             Some(x) => x as usize,
         };
         (start, stop.saturating_sub(start))
@@ -87,11 +105,36 @@ fn get_offset_and_len<F: FnOnce() -> usize>(
     }
 }
 
-pub fn slice(value: Value, start: Value, stop: Value, step: Value) -> Result<Value, Error> {
-    let start: i64 = if start.is_none() {
-        0
+fn range_step_backwards(
+    start: Option<i64>,
+    stop: Option<i64>,
+    step: usize,
+    end: usize,
+) -> impl Iterator<Item = usize> {
+    let start = match start {
+        None => end.saturating_sub(1),
+        Some(start) if start >= end as i64 => end.saturating_sub(1),
+        Some(start) if start >= 0 => start as usize,
+        Some(start) => (end as i64 + start).max(0) as usize,
+    };
+    let stop = match stop {
+        None => 0,
+        Some(stop) if stop < 0 => (end as i64 + stop).max(0) as usize,
+        Some(stop) => stop as usize,
+    };
+    let length = if stop == 0 {
+        (start + step) / step
     } else {
-        ok!(start.try_into())
+        (start - stop + step - 1) / step
+    };
+    (stop..=start).rev().step_by(step).take(length)
+}
+
+pub fn slice(value: Value, start: Value, stop: Value, step: Value) -> Result<Value, Error> {
+    let start = if start.is_none() {
+        None
+    } else {
+        Some(ok!(start.try_into()))
     };
     let stop = if stop.is_none() {
         None
@@ -99,9 +142,9 @@ pub fn slice(value: Value, start: Value, stop: Value, step: Value) -> Result<Val
         Some(ok!(i64::try_from(stop)))
     };
     let step = if step.is_none() {
-        1
+        1i64
     } else {
-        ok!(u64::try_from(step)) as usize
+        ok!(i64::try_from(step))
     };
     if step == 0 {
         return Err(Error::new(
@@ -113,39 +156,74 @@ pub fn slice(value: Value, start: Value, stop: Value, step: Value) -> Result<Val
     let kind = value.kind();
     let error = Err(Error::new(
         ErrorKind::InvalidOperation,
-        format!("value of type {} cannot be sliced", kind),
+        format!("value of type {kind} cannot be sliced"),
     ));
 
     match value.0 {
         ValueRepr::String(..) | ValueRepr::SmallStr(_) => {
             let s = value.as_str().unwrap();
-            let (start, len) = get_offset_and_len(start, stop, || s.chars().count());
-            Ok(Value::from(
-                s.chars()
-                    .skip(start)
-                    .take(len)
-                    .step_by(step)
-                    .collect::<String>(),
-            ))
+            if step > 0 {
+                let (start, len) = get_offset_and_len(start, stop, || s.chars().count());
+                Ok(Value::from(
+                    s.chars()
+                        .skip(start)
+                        .take(len)
+                        .step_by(step as usize)
+                        .collect::<String>(),
+                ))
+            } else {
+                let chars: Vec<char> = s.chars().collect();
+                Ok(Value::from(
+                    range_step_backwards(start, stop, -step as usize, chars.len())
+                        .map(move |i| chars[i])
+                        .collect::<String>(),
+                ))
+            }
         }
         ValueRepr::Bytes(ref b) => {
-            let (start, len) = get_offset_and_len(start, stop, || b.len());
-            Ok(Value::from_bytes(
-                b.get(start..start + len).unwrap_or_default().to_owned(),
-            ))
+            if step > 0 {
+                let (start, len) = get_offset_and_len(start, stop, || b.len());
+                Ok(Value::from_bytes(
+                    b.iter()
+                        .skip(start)
+                        .take(len)
+                        .step_by(step as usize)
+                        .copied()
+                        .collect(),
+                ))
+            } else {
+                Ok(Value::from_bytes(
+                    range_step_backwards(start, stop, -step as usize, b.len())
+                        .map(|i| b[i])
+                        .collect::<Vec<u8>>(),
+                ))
+            }
         }
         ValueRepr::Undefined(_) | ValueRepr::None => Ok(Value::from(Vec::<Value>::new())),
         ValueRepr::Object(obj) if matches!(obj.repr(), ObjectRepr::Seq | ObjectRepr::Iterable) => {
-            Ok(Value::make_object_iterable(obj, move |obj| {
+            if step > 0 {
                 let len = obj.enumerator_len().unwrap_or_default();
                 let (start, len) = get_offset_and_len(start, stop, || len);
-                // The manual match here is important that we do not mess up the size_hint
-                if let Some(iter) = obj.try_iter() {
-                    Box::new(iter.skip(start).take(len).step_by(step))
-                } else {
-                    Box::new(None.into_iter())
-                }
-            }))
+                Ok(Value::make_object_iterable(obj, move |obj| {
+                    if let Some(iter) = obj.try_iter() {
+                        Box::new(iter.skip(start).take(len).step_by(step as usize))
+                    } else {
+                        Box::new(None.into_iter())
+                    }
+                }))
+            } else {
+                Ok(Value::make_object_iterable(obj.clone(), move |obj| {
+                    if let Some(iter) = obj.try_iter() {
+                        let vec: Vec<Value> = iter.collect();
+                        Box::new(
+                            range_step_backwards(start, stop, -step as usize, vec.len())
+                                .map(move |i| vec[i].clone()),
+                        )
+                    } else {
+                        Box::new(None.into_iter())
+                    }
+                }))
+            }
         }
         _ => error,
     }
@@ -255,22 +333,6 @@ pub fn mul(lhs: &Value, rhs: &Value) -> Result<Value, Error> {
 }
 
 fn repeat_iterable(n: &Value, seq: &DynObject) -> Result<Value, Error> {
-    struct LenIterWrap<I: Send + Sync>(usize, I);
-
-    impl<I: Iterator<Item = Value> + Send + Sync> Iterator for LenIterWrap<I> {
-        type Item = Value;
-
-        #[inline(always)]
-        fn next(&mut self) -> Option<Self::Item> {
-            self.1.next()
-        }
-
-        #[inline(always)]
-        fn size_hint(&self) -> (usize, Option<usize>) {
-            (self.0, Some(self.0))
-        }
-    }
-
     let n = ok!(n.as_usize().ok_or_else(|| {
         Error::new(
             ErrorKind::InvalidOperation,
@@ -500,4 +562,183 @@ mod tests {
             Value::from("2342")
         );
     }
+
+    #[test]
+    fn test_slicing() {
+        let v = Value::from(vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9]);
+
+        // [::] - full slice
+        assert_eq!(
+            slice(v.clone(), Value::from(()), Value::from(()), Value::from(())).unwrap(),
+            Value::from(vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
+        );
+
+        // [::2] - every 2nd element
+        assert_eq!(
+            slice(v.clone(), Value::from(()), Value::from(()), Value::from(2)).unwrap(),
+            Value::from(vec![0, 2, 4, 6, 8])
+        );
+
+        // [1:2:2] - slice with start, stop, step
+        assert_eq!(
+            slice(v.clone(), Value::from(1), Value::from(2), Value::from(2)).unwrap(),
+            Value::from(vec![1])
+        );
+
+        // [::-2] - reverse with step of 2
+        assert_eq!(
+            slice(v.clone(), Value::from(()), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from(vec![9, 7, 5, 3, 1])
+        );
+
+        // [:-8:] - from index 0 to -8
+        assert_eq!(
+            slice(v.clone(), Value::from(()), Value::from(-8), Value::from(())).unwrap(),
+            Value::from(vec![0, 1])
+        );
+
+        // [-8::] - from index -8 to the end
+        assert_eq!(
+            slice(v.clone(), Value::from(-8), Value::from(()), Value::from(())).unwrap(),
+            Value::from(vec![2, 3, 4, 5, 6, 7, 8, 9])
+        );
+
+        // [-11::] - from index -11 to the end, which is the same as [::]
+        // because the start index is before the start of the vector
+        assert_eq!(
+            slice(
+                v.clone(),
+                Value::from(-11),
+                Value::from(()),
+                Value::from(())
+            )
+            .unwrap(),
+            Value::from(vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9])
+        );
+
+        // [:-11:] - from index -11 to the end, which is the same as [:0:]
+        // because the end index is before the start of the vector
+        assert_eq!(
+            slice(
+                v.clone(),
+                Value::from(()),
+                Value::from(-11),
+                Value::from(())
+            )
+            .unwrap(),
+            Value::from(Vec::<usize>::new())
+        );
+
+        // [2::-2] - from index 2 to start, reverse with step of 2
+        assert_eq!(
+            slice(v.clone(), Value::from(2), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from(vec![2, 0])
+        );
+
+        // [4:2:-2] - from index 4 to 2, reverse with step of 2
+        assert_eq!(
+            slice(v.clone(), Value::from(4), Value::from(2), Value::from(-2)).unwrap(),
+            Value::from(vec![4])
+        );
+
+        // [8:3:-2] - from index 8 to 3, reverse with step of 2
+        assert_eq!(
+            slice(v.clone(), Value::from(8), Value::from(3), Value::from(-2)).unwrap(),
+            Value::from(vec![8, 6, 4])
+        );
+    }
+
+    #[test]
+    fn test_string_slicing() {
+        let s = Value::from("abcdefghij");
+
+        // [::] - full slice
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(())).unwrap(),
+            Value::from("abcdefghij")
+        );
+
+        // [::2] - every 2nd character
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(2)).unwrap(),
+            Value::from("acegi")
+        );
+
+        // [1:2:2] - slice with start, stop, step
+        assert_eq!(
+            slice(s.clone(), Value::from(1), Value::from(2), Value::from(2)).unwrap(),
+            Value::from("b")
+        );
+
+        // [::-2] - reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from("jhfdb")
+        );
+
+        // [2::-2] - from index 2 to start, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(2), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from("ca")
+        );
+
+        // [4:2:-2] - from index 4 to 2, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(4), Value::from(2), Value::from(-2)).unwrap(),
+            Value::from("e")
+        );
+
+        // [8:3:-2] - from index 8 to 3, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(8), Value::from(3), Value::from(-2)).unwrap(),
+            Value::from("ige")
+        );
+    }
+
+    #[test]
+    fn test_bytes_slicing() {
+        let s = Value::from_bytes(b"abcdefghij".to_vec());
+
+        // [::] - full slice
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(())).unwrap(),
+            Value::from_bytes(b"abcdefghij".to_vec())
+        );
+
+        // [::2] - every 2nd character
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(2)).unwrap(),
+            Value::from_bytes(b"acegi".to_vec())
+        );
+
+        // [1:2:2] - slice with start, stop, step
+        assert_eq!(
+            slice(s.clone(), Value::from(1), Value::from(2), Value::from(2)).unwrap(),
+            Value::from_bytes(b"b".to_vec())
+        );
+
+        // [::-2] - reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(()), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from_bytes(b"jhfdb".to_vec())
+        );
+
+        // [2::-2] - from index 2 to start, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(2), Value::from(()), Value::from(-2)).unwrap(),
+            Value::from_bytes(b"ca".to_vec())
+        );
+
+        // [4:2:-2] - from index 4 to 2, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(4), Value::from(2), Value::from(-2)).unwrap(),
+            Value::from_bytes(b"e".to_vec())
+        );
+
+        // [8:3:-2] - from index 8 to 3, reverse with step of 2
+        assert_eq!(
+            slice(s.clone(), Value::from(8), Value::from(3), Value::from(-2)).unwrap(),
+            Value::from_bytes(b"ige".to_vec())
+        );
+    }
 }
