@@ -2,12 +2,14 @@ use proc_macro2::{Ident, Span, TokenStream};
 use quote::{format_ident, quote, quote_spanned, ToTokens};
 use syn::{parse::Error, parse_quote, punctuated::Punctuated, GenericArgument, ItemFn, Token};
 
-use crate::builder_attr::{IntoSetting, TypeBuilderAttr};
-use crate::field_info::FieldInfo;
-use crate::mutator::Mutator;
-use crate::util::{
-    empty_type, empty_type_tuple, first_visibility, modify_types_generics_hack, public_visibility, strip_raw_ident_prefix,
-    type_tuple,
+use crate::{
+    builder_attr::{IntoSetting, TypeBuilderAttr},
+    field_info::FieldInfo,
+    mutator::Mutator,
+    util::{
+        empty_type, empty_type_tuple, first_visibility, modify_types_generics_hack, phantom_data_for_generics, public_visibility,
+        strip_raw_ident_prefix, type_tuple,
+    },
 };
 
 #[derive(Debug)]
@@ -103,17 +105,7 @@ impl<'a> StructInfo<'a> {
         let generics_with_empty = modify_types_generics_hack(&ty_generics, |args| {
             args.push(syn::GenericArgument::Type(init_fields_type.clone().into()));
         });
-        let phantom_generics = self.generics.params.iter().filter_map(|param| match param {
-            syn::GenericParam::Lifetime(lifetime) => {
-                let lifetime = &lifetime.lifetime;
-                Some(quote!(&#lifetime ()))
-            }
-            syn::GenericParam::Type(ty) => {
-                let ty = &ty.ident;
-                Some(ty.to_token_stream())
-            }
-            syn::GenericParam::Const(_cnst) => None,
-        });
+        let phantom_data = phantom_data_for_generics(self.generics);
 
         let builder_method_name = self.builder_attr.builder_method.get_name().unwrap_or_else(|| quote!(builder));
         let builder_method_visibility = first_visibility(&[
@@ -193,7 +185,7 @@ impl<'a> StructInfo<'a> {
             #[allow(dead_code, non_camel_case_types, non_snake_case)]
             #builder_type_visibility struct #builder_name #b_generics #b_generics_where_extras_predicates {
                 fields: #all_fields_param,
-                phantom: ::core::marker::PhantomData<(#( ::core::marker::PhantomData<#phantom_generics> ),*)>,
+                phantom: #phantom_data,
             }
 
             #[automatically_derived]
@@ -212,15 +204,18 @@ impl<'a> StructInfo<'a> {
     fn field_impl(&self, field: &FieldInfo) -> syn::Result<TokenStream> {
         let StructInfo { ref builder_name, .. } = *self;
 
-        let descructuring = self.included_fields().map(|f| {
-            if f.ordinal == field.ordinal {
-                quote!(())
-            } else {
-                let name = f.name;
-                name.to_token_stream()
-            }
-        });
-        let reconstructing = self.included_fields().map(|f| f.name);
+        let destructuring = self
+            .included_fields()
+            .map(|f| {
+                if f.ordinal == field.ordinal {
+                    quote!(())
+                } else {
+                    let name = f.name;
+                    name.to_token_stream()
+                }
+            })
+            .collect::<Vec<_>>();
+        let reconstructing = self.included_fields().map(|f| f.name).collect::<Vec<_>>();
 
         let &FieldInfo {
             name: field_name,
@@ -251,16 +246,45 @@ impl<'a> StructInfo<'a> {
         target_generics.push(syn::GenericArgument::Type(target_generics_tuple.into()));
         ty_generics.push(syn::GenericArgument::Type(ty_generics_tuple.into()));
         let (impl_generics, _, where_clause) = generics.split_for_impl();
-        let doc = field.builder_attr.setter.doc.as_ref().map(|doc| quote!(#[doc = #doc]));
+        let doc = if let Some(doc) = field.builder_attr.setter.doc.as_ref() {
+            Some(quote!(#[doc = #doc]))
+        } else if !field.builder_attr.doc_comments.is_empty() {
+            Some(
+                field
+                    .builder_attr
+                    .doc_comments
+                    .iter()
+                    .map(|&line| quote!(#[doc = #line]))
+                    .collect(),
+            )
+        } else {
+            None
+        };
+
         let deprecated = &field.builder_attr.deprecated;
 
-        // NOTE: both auto_into and strip_option affect `arg_type` and `arg_expr`, but the order of
-        // nesting is different so we have to do this little dance.
+        let option_was_stripped;
         let arg_type = if field.builder_attr.setter.strip_option.is_some() && field.builder_attr.setter.transform.is_none() {
-            field
-                .type_from_inside_option()
-                .ok_or_else(|| Error::new_spanned(field_type, "can't `strip_option` - field is not `Option<...>`"))?
+            if let Some(inner_type) = field.type_from_inside_option() {
+                option_was_stripped = true;
+                inner_type
+            } else if field
+                .builder_attr
+                .setter
+                .strip_option
+                .as_ref()
+                .map_or(false, |s| s.ignore_invalid)
+            {
+                option_was_stripped = false;
+                field_type
+            } else {
+                return Err(Error::new_spanned(
+                    field_type,
+                    "can't `strip_option` - field is not `Option<...>`",
+                ));
+            }
         } else {
+            option_was_stripped = false;
             field_type
         };
         let (arg_type, arg_expr) = if field.builder_attr.setter.auto_into.is_some() {
@@ -269,13 +293,35 @@ impl<'a> StructInfo<'a> {
             (arg_type.to_token_stream(), field_name.to_token_stream())
         };
 
+        let strip_bool_fallback = field
+            .builder_attr
+            .setter
+            .strip_bool
+            .as_ref()
+            .and_then(|strip_bool| strip_bool.fallback.as_ref())
+            .map(|fallback| (fallback.clone(), quote!(#field_name: #field_type), quote!(#arg_expr)));
+
+        let strip_option_fallback = field.builder_attr.setter.strip_option.as_ref().and_then(|strip_option| {
+            if let Some(ref fallback) = strip_option.fallback {
+                Some((fallback.clone(), quote!(#field_name: #field_type), quote!(#arg_expr)))
+            } else if strip_option.fallback_prefix.is_none() && strip_option.fallback_suffix.is_none() {
+                None
+            } else {
+                let method = strip_raw_ident_prefix(field_name.to_string());
+                let prefix = strip_option.fallback_prefix.as_deref().unwrap_or_default();
+                let suffix = strip_option.fallback_suffix.as_deref().unwrap_or_default();
+                let fallback_name = syn::Ident::new(&format!("{}{}{}", prefix, method, suffix), field_name.span());
+                Some((fallback_name, quote!(#field_name: #field_type), quote!(#arg_expr)))
+            }
+        });
+
         let (param_list, arg_expr) = if field.builder_attr.setter.strip_bool.is_some() {
             (quote!(), quote!(true))
         } else if let Some(transform) = &field.builder_attr.setter.transform {
             let params = transform.params.iter().map(|(pat, ty)| quote!(#pat: #ty));
             let body = &transform.body;
             (quote!(#(#params),*), quote!({ #body }))
-        } else if field.builder_attr.setter.strip_option.is_some() {
+        } else if option_was_stripped {
             (quote!(#field_name: #arg_type), quote!(Some(#arg_expr)))
         } else {
             (quote!(#field_name: #arg_type), arg_expr)
@@ -293,6 +339,42 @@ impl<'a> StructInfo<'a> {
 
         let method_name = field.setter_method_name();
 
+        let strip_option_fallback_method = if let Some((method_name, param_list, arg_expr)) = strip_option_fallback {
+            Some(quote! {
+                #deprecated
+                #doc
+                #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
+                pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
+                    let #field_name = (#arg_expr,);
+                    let ( #(#destructuring,)* ) = self.fields;
+                    #builder_name {
+                        fields: ( #(#reconstructing,)* ),
+                        phantom: self.phantom,
+                    }
+                }
+            })
+        } else {
+            None
+        };
+
+        let strip_bool_fallback_method = if let Some((method_name, param_list, arg_expr)) = strip_bool_fallback {
+            Some(quote! {
+                #deprecated
+                #doc
+                #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
+                pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
+                    let #field_name = (#arg_expr,);
+                    let ( #(#destructuring,)* ) = self.fields;
+                    #builder_name {
+                        fields: ( #(#reconstructing,)* ),
+                        phantom: self.phantom,
+                    }
+                }
+            })
+        } else {
+            None
+        };
+
         Ok(quote! {
             #[allow(dead_code, non_camel_case_types, missing_docs)]
             #[automatically_derived]
@@ -302,12 +384,14 @@ impl<'a> StructInfo<'a> {
                 #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
                 pub fn #method_name (self, #param_list) -> #builder_name <#target_generics> {
                     let #field_name = (#arg_expr,);
-                    let ( #(#descructuring,)* ) = self.fields;
+                    let ( #(#destructuring,)* ) = self.fields;
                     #builder_name {
                         fields: ( #(#reconstructing,)* ),
                         phantom: self.phantom,
                     }
                 }
+                #strip_option_fallback_method
+                #strip_bool_fallback_method
             }
             #[doc(hidden)]
             #[allow(dead_code, non_camel_case_types, non_snake_case)]
@@ -320,6 +404,7 @@ impl<'a> StructInfo<'a> {
                 #[deprecated(
                     note = #repeated_fields_error_message
                 )]
+                #doc
                 pub fn #method_name (self, _: #repeated_fields_error_type_name) -> #builder_name <#target_generics> {
                     self
                 }
@@ -354,8 +439,8 @@ impl<'a> StructInfo<'a> {
             let mut generics = self.generics.clone();
             for f in self.included_fields() {
                 if f.builder_attr.default.is_some() || f.builder_attr.via_mutators.is_some() {
-                    // `f` is not mandatory - it does not have it's own fake `build` method, so `field` will need
-                    // to warn about missing `field` whether or not `f` is set.
+                    // `f` is not mandatory - it does not have its own fake `build` method, so `field` will need
+                    // to warn about missing `field` regardless of whether `f` is set.
                     assert!(
                         f.ordinal != field.ordinal,
                         "`required_field_impl` called for optional field {}",
@@ -455,8 +540,15 @@ impl<'a> StructInfo<'a> {
         let ItemFn { attrs, vis, .. } = mutator_fn;
         let sig = mutator.outer_sig(parse_quote!(#builder_name <#ty_generics>));
         let fn_name = &sig.ident;
+        let (_fn_impl_generics, fn_ty_generics, _fn_where_clause) = &sig.generics.split_for_impl();
+        let fn_call_turbofish = fn_ty_generics.as_turbofish();
         let mutator_args = mutator.arguments();
 
+        // Generics for the mutator - should be similar to the struct's generics
+        let m_generics = &self.generics;
+        let (m_impl_generics, m_ty_generics, m_where_clause) = m_generics.split_for_impl();
+        let m_phantom = phantom_data_for_generics(self.generics);
+
         Ok(quote! {
             #[allow(dead_code, non_camel_case_types, missing_docs)]
             #[automatically_derived]
@@ -464,25 +556,30 @@ impl<'a> StructInfo<'a> {
                 #(#attrs)*
                 #[allow(clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
                 #vis #sig {
-                    struct #mutator_struct_name {
+                    struct #mutator_struct_name #m_generics #m_where_clause {
+                        __phantom: #m_phantom,
                         #mutator_ty_fields
                     }
-                    impl #mutator_struct_name {
+                    impl #m_impl_generics #mutator_struct_name #m_ty_generics #m_where_clause {
                         #mutator_fn
                     }
 
                     let __args = (#mutator_args);
 
                     let ( #destructuring ) = self.fields;
-                    let mut __mutator = #mutator_struct_name{ #mutator_destructure_fields };
+                    let mut __mutator: #mutator_struct_name #m_ty_generics = #mutator_struct_name {
+                        __phantom: ::core::default::Default::default(),
+                        #mutator_destructure_fields
+                    };
 
                     // This dance is required to keep mutator args and destrucutre fields from interfering.
                     {
                         let (#mutator_args) = __args;
-                        __mutator.#fn_name(#mutator_args);
+                        __mutator.#fn_name #fn_call_turbofish(#mutator_args);
                     }
 
                     let #mutator_struct_name {
+                        __phantom,
                         #mutator_destructure_fields
                     } = __mutator;
 
@@ -556,7 +653,7 @@ impl<'a> StructInfo<'a> {
             ));
         });
 
-        let descructuring = self.included_fields().map(|f| f.name);
+        let destructuring = self.included_fields().map(|f| f.name);
 
         // The default of a field can refer to earlier-defined fields, which we handle by
         // writing out a bunch of `let` statements first, which can each refer to earlier ones.
@@ -619,7 +716,7 @@ impl<'a> StructInfo<'a> {
                 #build_method_doc
                 #[allow(clippy::default_trait_access, clippy::used_underscore_binding, clippy::no_effect_underscore_binding)]
                 #build_method_visibility fn #build_method_name #build_method_generic (self) -> #output_type #build_method_where_clause {
-                    let ( #(#descructuring,)* ) = self.fields;
+                    let ( #(#destructuring,)* ) = self.fields;
                     #( #assignments )*
 
                     #[allow(deprecated)]
